library kplayer;

import 'dart:io';

import 'package:flutter/foundation.dart';

import 'package:kplayer/src/_unimplement_dart_vlc.dart'
    if (dart.library.ffi) 'package:kplayer/src/_dart_vlc.dart';
import 'package:kplayer/src/_unimplement_just_audio.dart'
    if (dart.library.ffi)  'package:kplayer/src/_just_audio.dart';

enum PlayerStatus {
  uninitialized,
  initialized,
  playing,
  paused,
  stopped,
  completed
}

enum PlayerMediaType {
  asset,
  network,
  // memory,
}

// object handle Player resources
class PlayerMedia {
  PlayerMedia(this.type, this.resource);

  final PlayerMediaType type;
  final String resource;

  factory PlayerMedia.asset(String _resource) {
    return PlayerMedia(PlayerMediaType.asset, _resource);
  }
  factory PlayerMedia.network(String _resource) {
    return PlayerMedia(PlayerMediaType.network, _resource);
  }
}

/*
 * the player interface
*/
abstract class Player {
  Player(
      {required this.media,
      this.autoPlay = false,
      this.once = false,
      this.loop = false});

  factory Player.start(
      {int? id,
      required PlayerMedia media,
      required bool autoPlay,
      bool? once}) {
    if (!kIsWeb && (Platform.isWindows || Platform.isLinux)) {
      return DartVlcPlayer(
          id: id, media: media, autoPlay: autoPlay, once: true);
    } else {
      return JustAudioPlayer(media: media, autoPlay: autoPlay, once: true);
    }
  }
  factory Player.assets(media, {int? id, required bool autoPlay, bool? once}) {
    return Player.start(
        id: id, media: PlayerMedia.asset(media), autoPlay: autoPlay, once: once)
      ..init();
  }
  factory Player.network(media, {int? id, required bool autoPlay, bool? once}) {
    return Player.start(
        id: id,
        media: PlayerMedia.network(media),
        autoPlay: autoPlay,
        once: once)
      ..init();
  }
  static void boot() {
    DartVlcPlayer.boot();
    JustAudioPlayer.boot();
  }

  final bool autoPlay;
  final bool once;
  final bool loop;
  final PlayerMedia media;

  static List palyers = <Player>[];
  void init();
  void play();
  void pause();
  void stop();
  void seek(Duration position);
  void toggle(Duration position);
  void setVolume(double volume);
  void setSpeed(double rate);
  void dispose();

  get package;
  bool get playing;
  Duration get position;
  Duration get duration;
  PlayerStatus get status;
}
