import 'dart:io';

import 'package:dart_vlc/dart_vlc.dart' as dart_vlc;
import 'package:flutter/foundation.dart';
import 'package:kplayer/kplayer.dart';

// dart_vlc wrapper
class DartVlcPlayer extends Player {
  DartVlcPlayer(
      {int? id,
      required PlayerMedia media,
      required autoPlay,
      bool once = false,
      bool loop = false})
      : player = dart_vlc.Player(id: id ?? Player.palyers.length + 50001),
        super(media: media, autoPlay: autoPlay, once: once, loop: loop);

  final dart_vlc.Player player;

  @override
  get package => "dart_vlc";

  PlayerStatus _status = PlayerStatus.uninitialized;

  @override
  PlayerStatus get status => _status;
  Duration _position = Duration.zero;
  Duration _duration = Duration.zero;

  @override
  bool get playing => _status == PlayerStatus.playing;

  @override
  Duration get duration => _duration;

  @override
  Duration get position => _position;

  @override
  void init() {
    var _vlcMedia = media.type == PlayerMediaType.asset
        ? dart_vlc.Media.asset(media.resource)
        : dart_vlc.Media.network(media.resource);
    player.open(_vlcMedia, autoStart: autoPlay);
    _status = PlayerStatus.initialized;
    Player.palyers.add(this);

    player.positionStream.listen((dart_vlc.PositionState state) {
      _position = state.position ?? _position;
      _duration = state.duration ?? _duration;
    });
  }

  @override
  void play() {
    player.play();
    _status = PlayerStatus.playing;
    if (once) {
      player.playbackStream.listen((event) {
        if (event.isCompleted) {
          dispose();
        }
      });
    }
  }

  @override
  void pause() {
    player.pause();
    _status = PlayerStatus.paused;
  }

  @override
  void stop() {
    player.stop();
    _status = PlayerStatus.stopped;
  }

  @override
  void dispose() {
    player.dispose();
  }

  @override
  void seek(Duration position) {
    player.seek(position);
  }

  @override
  void toggle(Duration position) {
    player.playOrPause();
    // player. ? pause() : play();
  }

  @override
  void setVolume(double volume) {
    player.setVolume(0.5);
  }

  @override
  void setSpeed(double rate) {
    player.setRate(rate);
  }

  void boot() {
    if (!kIsWeb && (Platform.isWindows || Platform.isLinux)) {
      dart_vlc.DartVLC.initialize();
    }
  }
}
