import 'package:kplayer/kplayer.dart';
import 'package:just_audio/just_audio.dart' as just_audio;

// just_audio wrapper
class JustAudioPlayer extends Player {
  JustAudioPlayer(
      {required PlayerMedia media, required autoPlay, once, bool loop = false})
      : player = just_audio.AudioPlayer(),
        super(media: media, autoPlay: autoPlay, once: once, loop: loop);

  final just_audio.AudioPlayer player;
  @override
  get package => "just_audio";

  PlayerStatus _status = PlayerStatus.uninitialized;
  @override
  PlayerStatus get status => _status;

  @override
  bool get playing => _status == PlayerStatus.playing;

  @override
  Duration get duration => player.duration ?? Duration.zero;

  @override
  Duration get position => player.position;

  @override
  void init() async {
    if (media.type == PlayerMediaType.asset) {
      await player.setAsset(media.resource);
    } else {
      await player.setUrl(media.resource);
    }
    _status = PlayerStatus.initialized;
    Player.palyers.add(this);
    if (autoPlay) {
      play();
    }
  }

  @override
  void play() {
    player.play();
    _status = PlayerStatus.playing;
    if (once) {
      player.playerStateStream.listen((event) {
        if (event.processingState == just_audio.ProcessingState.completed) {
          dispose();
        }
      });
    }
  }

  @override
  void pause() {
    player.pause();
    _status = PlayerStatus.paused;
  }

  @override
  void stop() {
    player.stop();
    _status = PlayerStatus.stopped;
  }

  @override
  void seek(Duration position) {
    player.seek(position);
  }

  @override
  void toggle(Duration position) {
    player.playing ? pause() : play();
  }

  @override
  void setVolume(double volume) {
    player.setVolume(0.5);
  }

  @override
  void setSpeed(double rate) {
    player.setSpeed(rate);
  }

  @override
  void dispose() {
    player.dispose();
  }

  static void boot() {
    print("Kplayer: just_audio");
  }
}
